package com.pixelmintdrop.model

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.activity.result.ActivityResultLauncher
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.games.PlayGames
import com.google.android.gms.games.PlayGamesSdk
import com.google.android.gms.tasks.Task

class GooglePlayGamesManager(private val context: Context) {
    private val TAG = "GooglePlayGamesManager"
    
    // Define a listener interface for the leaderboard intent
    interface LeaderboardIntentListener {
        fun onLeaderboardIntentSuccess(intent: Intent)
        fun onLeaderboardIntentFailure(exception: Exception)
    }
    
    // Leaderboard ID
    companion object {
        const val LEADERBOARD_ID = "CgkImJW2mKsSEAIQAQ"
    }
    
    // Initialize the Play Games SDK
    init {
        try {
            PlayGamesSdk.initialize(context)
            Log.d(TAG, "PlayGamesSdk initialized")
        } catch (e: Exception) {
            Log.e(TAG, "Error initializing PlayGamesSdk", e)
        }
    }
    
    // Check if user is already signed in
    fun isSignedIn(): Boolean {
        val account = GoogleSignIn.getLastSignedInAccount(context)
        return account != null && !account.isExpired
    }
    
    // Get the sign-in client
    fun getSignInClient(): GoogleSignInClient {
        val signInOptions = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN).build()
        return GoogleSignIn.getClient(context, signInOptions)
    }
    
    // Handle sign-in result
    fun handleSignInResult(task: Task<GoogleSignInAccount>): Boolean {
        return if (task.isSuccessful) {
            Log.d(TAG, "Sign-in successful")
            true
        } else {
            Log.e(TAG, "Sign-in failed", task.exception)
            false
        }
    }
    
    // Submit score to leaderboard (requires an activity for potential sign-in)
    fun submitScore(score: Long, activity: Activity? = null) {
        if (activity == null) {
            Log.w(TAG, "Activity context is required for score submission.")
            return
        }

        Log.d(TAG, "Attempting to submit score: $score")
        
        // Always force a sign-in to ensure we have a valid account
        val signInClient = PlayGames.getGamesSignInClient(activity)
        signInClient.signIn().addOnCompleteListener { signInTask ->
            if (signInTask.isSuccessful) {
                Log.d(TAG, "Sign-in successful, retrieving account for score submission")
                
                // After sign-in, get the account and submit score
                try {
                    // Use PlayGames API directly to get client and submit score
                    val leaderboardsClient = PlayGames.getLeaderboardsClient(activity)
                    leaderboardsClient.submitScore(LEADERBOARD_ID, score)
                    Log.d(TAG, "Successfully submitted score $score to leaderboard $LEADERBOARD_ID")
                    
                    // Show confirmation to user
                    android.widget.Toast.makeText(
                        activity,
                        "Score submitted to leaderboard",
                        android.widget.Toast.LENGTH_SHORT
                    ).show()
                } catch (e: Exception) {
                    Log.e(TAG, "Error submitting score after sign-in", e)
                    android.widget.Toast.makeText(
                        activity,
                        "Failed to submit score to leaderboard",
                        android.widget.Toast.LENGTH_SHORT
                    ).show()
                }
            } else {
                Log.e(TAG, "Sign-in failed, unable to submit score", signInTask.exception)
                android.widget.Toast.makeText(
                    activity,
                    "Sign-in required to submit score to leaderboard",
                    android.widget.Toast.LENGTH_SHORT
                ).show()
            }
        }
    }
    
    // Show the leaderboard - Now uses a listener to return the Intent
    fun showLeaderboard(activity: Activity, listener: LeaderboardIntentListener) {
        Log.d(TAG, "showLeaderboard called in GooglePlayGamesManager")
        
        // First check if already authenticated
        PlayGames.getGamesSignInClient(activity).isAuthenticated.addOnCompleteListener { authTask ->
            val isAuthenticated = authTask.isSuccessful && authTask.result.isAuthenticated
            
            if (isAuthenticated) {
                // Already authenticated, get leaderboard
                getLeaderboardIntent(activity, listener)
            } else {
                // Need to authenticate first
                Log.d(TAG, "User not authenticated for leaderboard, attempting sign-in")
                PlayGames.getGamesSignInClient(activity).signIn().addOnCompleteListener { signInTask ->
                    if (signInTask.isSuccessful) {
                        // Successfully signed in, now get leaderboard
                        Log.d(TAG, "Sign-in successful, getting leaderboard")
                        getLeaderboardIntent(activity, listener)
                    } else {
                        // Failed to sign in
                        Log.e(TAG, "Failed to sign in for leaderboard access", signInTask.exception)
                        listener.onLeaderboardIntentFailure(
                            Exception("Sign-in required to view leaderboard", signInTask.exception)
                        )
                        
                        // Inform user
                        android.widget.Toast.makeText(
                            activity,
                            "Sign-in required to view leaderboard",
                            android.widget.Toast.LENGTH_SHORT
                        ).show()
                    }
                }
            }
        }
    }
    
    // Private helper to get leaderboard intent
    private fun getLeaderboardIntent(activity: Activity, listener: LeaderboardIntentListener) {
        Log.d(TAG, "Attempting to get LeaderboardsClient")
        try {
            val leaderboardsClient = PlayGames.getLeaderboardsClient(activity)
            Log.d(TAG, "LeaderboardsClient obtained, attempting to get leaderboard intent for ID: $LEADERBOARD_ID")
            leaderboardsClient
                .getLeaderboardIntent(LEADERBOARD_ID)
                .addOnSuccessListener { intent ->
                    Log.d(TAG, "Successfully obtained leaderboard intent. Calling listener.")
                    listener.onLeaderboardIntentSuccess(intent)
                }
                .addOnFailureListener { e ->
                    Log.e(TAG, "Error getting leaderboard intent", e)
                    listener.onLeaderboardIntentFailure(e)
                }
        } catch (e: Exception) {
            Log.e(TAG, "Error obtaining LeaderboardsClient or getting leaderboard intent", e)
            listener.onLeaderboardIntentFailure(e)
        }
    }
} 